// -------------------------------------------------------------------
// -----------------------LPC2106 I2C routines------------------------
// -------------------------------------------------------------------
// Written for TinyARM MB40 development board
// -------------------------------------------------------------------
// THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// -------------------------------------------------------------------

/*
*****************************************************************************
**
**      Project     : My project
**
**      Component   : LPC2106 (LPC2106)
**
**      Modulename  : I2C
**
**      Filename    : ma_i2c.c
**
**      Abstract    : This file implements a device driver for the I2C 
**                    module.
**
**      Compiler    : IAR C compiler
**
**      Date        : 2004-05-26 17:21:26
**
**      License no. : 9503-663-863-6224     Ivan
**
**      Warning     : This file has been automatically generated.
**                    Do not edit this file if you intend to regenerate it.
**
**      This device driver was created by IAR MakeApp version 
**      4.02A (Philips LPC210x: 4.00C) for the Philips LPC210x series of
**      microcontrollers.
**
**      (c)Copyright 2004 IAR Systems.
**      Your rights to this file are explained in the IAR MakeApp 
**      License Agreement. All other rights reserved.
**
*****************************************************************************
*/

/*
**===========================================================================
**  1       GENERAL
**  1.1     Revisions
**
**  Please read the IAR MakeApp for Philips LPC210x readme file 
**  
**
**===========================================================================
*/

/*
**===========================================================================
**  1.2     References
** 
**  No   Identification          Name or Description
**  ==   ===================     ================================
**
**  1    02/Oct/2003             Philips LPC210x Hardware Manual
** 
**===========================================================================
*/

/*
**===========================================================================
**  2.      INCLUDE FILES
**  2.1     Standard include files
**===========================================================================
*/

/*
**===========================================================================
**  2.2     Application include files
**===========================================================================
*/

#include "usercode.h"   /* Usercode macros (see <template.h>) */
#include "ma_tgt.h"     /* Target specific header file */
#include "ma_sfr.h"     /* Special function register bitfield macros */
#include "iolpc210x.h"  /* Defines Special function registers */

#include "ma_i2c.h"     /* Module driver header file */

/*
**===========================================================================
**  3.      DECLARATIONS
**  3.1     Internal constants
**===========================================================================
*/

#define MA_I2CONSET_I2C        0x00000040  /* I2C Control Set Register */
#define MA_I2CONSET_I2C_MASK   0x0000007C  /* Used bits */
#define MA_I2DAT_I2C           0x00000000  /* I2C Data Register */
#define MA_I2DAT_I2C_MASK      0x000000FF  /* Used bits */
#define MA_I2ADR_I2C           0x00000000  /* I2C Slave Address Register */
#define MA_I2ADR_I2C_MASK      0x000000FF  /* Used bits */
#define MA_I2SCLH_I2C          0x00000004  /* I2C SCL Duty Cycle High Register */
#define MA_I2SCLH_I2C_MASK     0x0000FFFF  /* Used bits */
#define MA_I2SCLL_I2C          0x00000004  /* I2C SCL Duty Cycle Low Register */
#define MA_I2SCLL_I2C_MASK     0x0000FFFF  /* Used bits */

// Ivan
char Info;
char StatusI2C;
// Ivan end
/*
**===========================================================================
**  3.2     Internal macros
**===========================================================================
*/

/*
**===========================================================================
**  3.3     Internal type definitions
**===========================================================================
*/

/*
**===========================================================================
**  3.4     Global variables (declared as 'extern' in some header file)
**===========================================================================
*/

/*
**===========================================================================
**  3.5     Internal function prototypes (defined in Section 5)
**===========================================================================
*/

/*
**===========================================================================
**  3.6     Internal variables
**===========================================================================
*/

/*
**===========================================================================
**  4.      GLOBAL FUNCTIONS (declared as 'extern' in some header file)
**===========================================================================
*/



void MA_Init_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**    Initialises the I2C module. Only sets those registers with values 
**    not equal to the power-on reset values. 
**
**  Parameters:
**      None
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{
    /*--- Handle user code on function entry ---*/
    ENTER_MA_INIT_I2C;

    /*--- Initialise registers ---*/
    I2CONSET = ( I2CONSET & ~MA_I2CONSET_I2C_MASK ) | MA_I2CONSET_I2C;

    /*--- Handle user code on function exit ---*/
    EXIT_MA_INIT_I2C;
    
} /* MA_Init_I2C */

    



void MA_Reset_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**    Resets the I2C module. Sets all registers.
**
**  Parameters:
**      None
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{
    /*--- Handle user code on function entry ---*/
    ENTER_MA_RESET_I2C;
    
    /*--- Clear flags ---*/
    I2CONCLR = 0x6c;    

    /*--- Reset registers ---*/
    I2SCLL   = ( I2SCLL & ~MA_I2SCLL_I2C_MASK ) | MA_I2SCLL_I2C;
    I2SCLH   = ( I2SCLH & ~MA_I2SCLH_I2C_MASK ) | MA_I2SCLH_I2C;
    I2ADR    = ( I2ADR & ~MA_I2ADR_I2C_MASK ) | MA_I2ADR_I2C;
    I2CONSET = ( I2CONSET & ~MA_I2CONSET_I2C_MASK ) | MA_I2CONSET_I2C;

    /*--- Handle user code on function exit ---*/
    EXIT_MA_RESET_I2C;
   
} /* MA_Reset_I2C */




S8 MA_Start_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Generates a start condition on I2C when bus is free. 
**      Master mode will also automatically be entered.
**
**      Note: After a stop condition, you may need a bus free time 
**            before you can generate a new start condition.
**
**  Parameters:
**      None
**
**  Returns:
**      MA_OK or I2C status code
**
**---------------------------------------------------------------------------
*/
{
    U8 Status, Flag;
    S8 ReturnCode;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_START_I2C;

    /*--- Issue a start condition ---*/
    I2CONSET_bit.STA = 1;      /* Set Start flag */ 

    /*--- Wait until START transmitted ---*/
    Flag = TRUE;
    while( Flag )
    {
        /*--- Get new status ---*/
        Status = MA_CheckStatus_I2C();

        if(( Status == 0x08 ) || ( Status == 0x10 ))
        {
            /*--- Start condition has been transmitted ---*/
            Flag = FALSE;
            ReturnCode = MA_OK;
        }
        else if( Status != 0xf8 )
        {
            /*--- ERROR ---*/
            Flag = FALSE;
            ReturnCode = (S8) Status;
        }
        else
        {
            /*--- Clear SI flag ---*/ 
            I2CONCLR = 0x08;    
        }
    }
    
    /*--- Clear Start flag ---*/ 
    I2CONCLR = 0x20;    

    /*--- Handle user code on function exit ---*/
    EXIT_MA_START_I2C;

    return ReturnCode;
    
} /* MA_Start_I2C */




S8 MA_RepeatStart_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Generates a repeated start condition on the I2C bus. 
**      Master mode will also automatically be entered.
**
**  Parameters:
**      None
**
**  Returns:
**      MA_OK or I2C status code
**
**---------------------------------------------------------------------------
*/
{
    U8 Status, Flag;
    S8 ReturnCode;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_REPEATSTART_I2C;

    /*--- Issue a start condition ---*/
    I2CONSET_bit.STA = 1;      /* Set Start flag */ 
    I2CONCLR = 0x08;  /* Clear SI flag */     

    /*--- Wait until START transmitted ---*/
    Flag = TRUE;
    while( Flag )
    {
        /*--- Get new status ---*/
        Status = MA_CheckStatus_I2C();

        if(( Status == 0x08 ) || ( Status == 0x10 ))
        {
            /*--- Start condition has been transmitted ---*/
            Flag = FALSE;
            ReturnCode = MA_OK;
        }
        else if( Status != 0xf8 )
        {
            /*--- ERROR ---*/
            Flag = FALSE;
            ReturnCode = (S8) Status;
        }
        else
        {
            /*--- Clear SI flag ---*/ 
            I2CONCLR = 0x08;    
        }
    }
    
    /*--- Clear Start flag ---*/ 
    I2CONCLR = 0x20;    

    /*--- Handle user code on function exit ---*/
    EXIT_MA_REPEATSTART_I2C;

    return ReturnCode;
    
} /* MA_RepeatStart_I2C */




S8 MA_Stop_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Generates a stop condition in master mode or recovers from an
**      error condition in slave mode.
**
**      Note: After this function is run, you may need a bus free time 
**            before you can generate a new start condition.
**
**  Parameters:
**      None
**
**  Returns:
**      MA_OK
**
**---------------------------------------------------------------------------
*/
{
    /*--- Handle user code on function entry ---*/
    ENTER_MA_STOP_I2C;

    I2CONSET_bit.STO = 1;      /* Set Stop flag */ 
    I2CONCLR = 0x08;  /* Clear SI flag */ 
            
    /*--- Wait for STOP detected ---*/
    while(            I2CONSET_bit.STO            == 1 )
    {
        /*--- do nothing ---*/
        ;
    }

    /*--- Handle user code on function exit ---*/
    EXIT_MA_STOP_I2C;

    return MA_OK;
    
} /* MA_Stop_I2C */





S8 MA_MasterPutChar_I2C( U8 Data ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Sends a character on the I2C network.
**
**  Parameters:
**      Data        Data byte to send
**
**  Returns:
**      MA_BUSY     If previous byte not sent
**      MA_OK       Otherwise
**
**---------------------------------------------------------------------------
*/
{
    S8 ReturnCode;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_MASTERPUTCHAR_I2C;
        
    /*--- Check if I2C Data register can be accessed ---*/
    if(         I2CONSET_bit.SI         )
    {
        /*--- Send data ---*/
        I2DAT = Data;
        I2CONCLR = 0x08;     /* Clear SI flag */ 
        ReturnCode = MA_OK;
    }
    else
    {
        /*--- Data register not ready ---*/
        ReturnCode = MA_BUSY;
    }

    /*--- Handle user code on function exit ---*/
    EXIT_MA_MASTERPUTCHAR_I2C;

    return ReturnCode;
    
} /* MA_MasterPutChar_I2C */





S8 MA_MasterGetChar_I2C( U8 Mode, U8 *pData ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Receives a character. I2C master mode used.
**      This function is also used to prepare if the master shall generate
**      acknowledge or not acknowledge.
**
**  Parameters:
**      Mode        = MA_I2C_ACK0   Set ACK=0. Slave sends next byte.
**                  = MA_I2C_ACK1   Set ACK=1. Slave sends last byte.
**                  = MA_I2C_READ   Read received data from data register.
**      pData       A pointer to where the received data shall be saved.
**
**  Returns:
**      MA_EMPTY    If no data available
**      MA_OK       Otherwise
**
**---------------------------------------------------------------------------
*/
{
    S8 ReturnCode;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_MASTERGETCHAR_I2C;

    ReturnCode = MA_OK;

    if( Mode == MA_I2C_ACK0 )
    {
        /*--- The operation mode is changed from master transmit to master receive ---*/
        /*--- Set ACK=0 (informs slave to send next byte) ---*/
        I2CONSET_bit.AA = 1;       
        I2CONCLR = 0x08;     /* Clear SI flag */   
    }
    else if( Mode == MA_I2C_ACK1 )
    {
        /*--- Set ACK=1 (informs slave to send last byte) ---*/
        I2CONCLR = 0x04;     
        I2CONCLR = 0x08;     /* Clear SI flag */ 
    }
    else if( Mode == MA_I2C_READ )
    {            
        /*--- Check if I2C Data register can be accessed ---*/
        if(             I2CONSET_bit.SI             )
        {
            /*--- Read data ---*/
            *pData = (U8) I2DAT;
        }
        else
        {
            /*--- No data available ---*/
            ReturnCode = MA_EMPTY;
        }
    }

    /*--- Handle user code on function exit ---*/
    EXIT_MA_MASTERGETCHAR_I2C;

    return ReturnCode;
    
} /* MA_MasterGetChar_I2C */





S8 MA_MasterWrite_I2C( U8 *pTxData, U16 Length ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Sends the specified number of characters on the I2C network.
**
**      Note: After this function is run, you may need a bus free time 
**            before a new data transfer can be initiated.
**
**  Parameters:
**      pTxData     Pointer to the data to transmit
**      Length      The number of data bytes to transmit
**
**  Returns:
**      MA_ERROR    If an error occured
**      MA_OK       Otherwise
**
**---------------------------------------------------------------------------
*/
{
    S8 ReturnCode;
    U8 Status, Flag, ii;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_MASTERWRITE_I2C;

	Info=0;
	StatusI2C=0;

    /*--- Generate Start condition ---*/
    ReturnCode = MA_Start_I2C();

    /*--- Transmit address ---*/
    if( ReturnCode == MA_OK )
    {
        /*--- Write SLA+W ---*/
        ReturnCode = MA_MasterPutChar_I2C( *pTxData );
        while( ReturnCode == MA_BUSY )
        {
            ReturnCode = MA_MasterPutChar_I2C( *pTxData );
        }
        pTxData++;
    }

    if( ReturnCode == MA_OK )
    {
        /*--- Wait until address transmitted and transmit data ---*/
        for( ii = 1; ii<Length; ii++ )
        {
            /*--- Wait until data transmitted ---*/
            Flag = TRUE;
            while( Flag )
            {
                /*--- Get new status ---*/
                Status = MA_CheckStatus_I2C();

                if( (Status == 0x18) || (Status == 0x28) )
                {
                    /*--- Data transmitted and ACK received ---*/
                    Flag = FALSE;

                    /*--- Write data ---*/
                    ReturnCode = MA_MasterPutChar_I2C( *pTxData );
                    while( ReturnCode == MA_BUSY )
                    {
                        ReturnCode = MA_MasterPutChar_I2C( *pTxData );
                    }
                    pTxData++;
                }
                else if( Status != 0xf8 )
                {
                    /*--- ERROR ---*/
                    Flag = FALSE;
                    ii = Length;
                    ReturnCode = MA_ERROR;
					Info|=1;
					StatusI2C=Status;
                }
            }
        }
    }

    /*--- Wait until data transmitted ---*/
    Flag = TRUE;
    while( Flag )
    {
        /*--- Get new status ---*/
        Status = MA_CheckStatus_I2C();

        if( (Status == 0x18) || (Status == 0x28) )
        {
            /*--- Data transmitted and ACK received ---*/
            Flag = FALSE;
        }
        else if( Status != 0xf8 )
        {
            /*--- ERROR ---*/
            Flag = FALSE;
            ii = Length;
			Info|=2;
            ReturnCode = MA_ERROR;
        }
    }

    /*--- Generate Stop condition ---*/
    MA_Stop_I2C();

    /*--- Handle user code on function exit ---*/
    EXIT_MA_MASTERWRITE_I2C;
  
    return ReturnCode;
    
} /* MA_MasterWrite_I2C */





S8 MA_MasterRead_I2C( U8 *pRxData, U16 Length ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Receives the specified number of characters on the I2C network.
**
**      Note1: Before using this function, the first byte in the receive
**      buffer must be loaded with slave address and direction bit (SLA+R).
**      The received data from the addressed slave will be saved in the 
**      buffer starting at the second byte.
**
**      Note2: After this function is run, you may need a bus free time 
**             before a new data transfer can be initiated.
**
**  Parameters:
**      pRxData     A pointer to the receive buffer
**      Length      The number of data bytes to receive
**
**  Returns:
**      MA_OK or I2C status code
**
**---------------------------------------------------------------------------
*/
{
    S8 ReturnCode;
    U8 Status, Flag, ii;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_MASTERREAD_I2C;

    /*--- Generate Start condition ---*/
    ReturnCode = MA_Start_I2C();

    /*--- Transmit address ---*/
    if( ReturnCode == MA_OK )
    {
        /*--- Write SLA+R ---*/
        ReturnCode = MA_MasterPutChar_I2C( *pRxData );
        while( ReturnCode == MA_BUSY )
        {
            ReturnCode = MA_MasterPutChar_I2C( *pRxData );
        }
        pRxData++;
    }

    if( ReturnCode == MA_OK )
    {
        /*--- wait until address transmitted and receive data ---*/
        for( ii = 1; ii<= Length; ii++ )
        {
            /*--- wait until data transmitted ---*/
            Flag = TRUE;
            while( Flag )
            {
                /*--- Get new status ---*/
                Status = MA_CheckStatus_I2C();

                if(( Status == 0x40 ) || ( Status == 0x48 ) || ( Status == 0x50 ))
                {
                    /*--- Data received ---*/
                    Flag = FALSE;

                    if( ii == Length )
                    {
                        /*--- Set generate NACK ---*/
                        ReturnCode = MA_MasterGetChar_I2C( MA_I2C_ACK1, pRxData );
                    }
                    else
                    {
                        ReturnCode = MA_MasterGetChar_I2C( MA_I2C_ACK0, pRxData );
                    }

                    /*--- Read data ---*/
                    ReturnCode = MA_MasterGetChar_I2C( MA_I2C_READ, pRxData );
                    while( ReturnCode == MA_EMPTY )
                    {
               	        ReturnCode = MA_MasterGetChar_I2C( MA_I2C_READ, pRxData );
                    }
                    pRxData++;
                }
                else if( Status != 0xf8 )
                {
                    /*--- ERROR ---*/
                    Flag = FALSE;
                    ii = Length;
                    ReturnCode = MA_ERROR;
                }
            }
        }
    }

    /*--- Generate Stop condition ---*/
    MA_Stop_I2C();

    /*--- Handle user code on function exit ---*/
    EXIT_MA_MASTERREAD_I2C;

    return ReturnCode;
    
} /* MA_MasterRead_I2C */




U8 MA_CheckStatus_I2C( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Checks the I2C status.
**
**  Parameters:
**      None
**
**  Returns:
**      00h Bus error
**      08h START condition transmitted
**      10h Repeated START condition transmitted
**      18h SLA + W transmitted, ACK received
**      20h SLA + W transmitted, ACK not received
**      28h Data byte transmitted, ACK received
**      30h Data byte transmitted, ACK not received
**      38h Arbitration lost
**      40h SLA + R transmitted, ACK received
**      48h SLA + R transmitted, ACK not received
**      50h Data byte received in master mode, ACK transmitted
**      58h Data byte received in master mode, ACK not transmitted
**      60h SLA + W received, ACK transmitted
**      68h Arbitration lost, SLA + W received, ACK transmitted
**      70h General call address received, ACK transmitted
**      78h Arbitration lost, general call addr received, ACK transmitted
**      80h Data byte received with own SLA, ACK transmitted
**      88h Data byte received with own SLA, ACK not transmitted
**      90h Data byte received after general call, ACK transmitted
**      98h Data byte received after general call, ACK not transmitted
**      A0h STOP or repeated START condition received in slave mode
**      A8h SLA + R received, ACK transmitted
**      B0h Arbitration lost, SLA + R received, ACK transmitted
**      B8h Data byte transmitted in slave mode, ACK received
**      C0h Data byte transmitted in slave mode, ACK not received
**      C8h Last byte transmitted in slave mode, ACK received
**      F8h No relevant status information, SI=0
**      FFh Channel error
**
**---------------------------------------------------------------------------
*/
{
    U8 Status;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_CHECKSTATUS_I2C;

    /*--- Wait for I2C Status changed ---*/
    while(            I2CONSET_bit.SI            == 0)
    {
        ;
    }
    
    /*--- Read I2C State ---*/
    Status = I2STAT;

    /*--- NOTE! SI flag is not cleared here ---*/

    /*--- Handle user code on function exit ---*/
    EXIT_MA_CHECKSTATUS_I2C;

    return Status;

} /* MA_CheckStatus_I2C */



/*
**===========================================================================
**  5.      INTERNAL FUNCTIONS (declared in Section 3.5)
**===========================================================================
*/

/*
**===========================================================================
** END OF FILE
**===========================================================================
*/ 



