// -------------------------------------------------------------------
// -----------------------LPC2106 UART0 routines----------------------
// -------------------------------------------------------------------
// Written for TinyARM MB40 development board
// -------------------------------------------------------------------
// THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// -------------------------------------------------------------------

/*
*****************************************************************************
**
**      Project     : My project
**
**      Component   : LPC2106 (LPC2106)
**
**      Modulename  : UART0
**
**      Filename    : ma_uart0.c
**
**      Abstract    : This file implements a device driver for the UART0 
**                    module.
**
**      Compiler    : IAR C compiler
**
**      Date        : 2004-08-23 19:00:54
**
**      License no. : 9503-663-863-6224     Ivan
**
**      Warning     : This file has been automatically generated.
**                    Do not edit this file if you intend to regenerate it.
**
**      This device driver was created by IAR MakeApp version 
**      4.02A (Philips LPC210x: 4.00C) for the Philips LPC210x series of
**      microcontrollers.
**
**      (c)Copyright 2004 IAR Systems.
**      Your rights to this file are explained in the IAR MakeApp 
**      License Agreement. All other rights reserved.
**
*****************************************************************************
*/

/*
**===========================================================================
**  1       GENERAL
**  1.1     Revisions
**
**  Please read the IAR MakeApp for Philips LPC210x readme file 
**  
**
**===========================================================================
*/

/*
**===========================================================================
**  1.2     References
** 
**  No   Identification          Name or Description
**  ==   ===================     ================================
**
**  1    02/Oct/2003             Philips LPC210x Hardware Manual
** 
**===========================================================================
*/

/*
**===========================================================================
**  2.      INCLUDE FILES
**  2.1     Standard include files
**===========================================================================
*/

#include <inarm.h> 

/*
**===========================================================================
**  2.2     Application include files
**===========================================================================
*/

#include "usercode.h"   /* Usercode macros (see <template.h>) */
#include "ma_tgt.h"     /* Target specific header file */
#include "ma_sfr.h"     /* Special function register bitfield macros */
#include "iolpc210x.h"  /* Defines Special function registers */

#include "ma_uart0.h"    /* Module driver header file */

/*
**===========================================================================
**  3.      DECLARATIONS
**  3.1     Internal constants
**===========================================================================
*/

#define MA_U0THR_UART0         0x00000000  /* UART0 Transmit Holding Register */
#define MA_U0THR_UART0_MASK    0x000000FF  /* Used bits */
#define MA_U0DLL_UART0         0x00000010  /* UART0 Divisor Latch LSB */
#define MA_U0DLL_UART0_MASK    0x000000FF  /* Used bits */
#define MA_U0DLM_UART0         0x00000000  /* UART0 Divisor Latch MSB */
#define MA_U0DLM_UART0_MASK    0x000000FF  /* Used bits */
#define MA_U0IER_UART0         0x00000000  /* UART0 Interrupt Enable Register */
#define MA_U0IER_UART0_MASK    0x00000007  /* Used bits */
#define MA_U0FCR_UART0         0x00000000  /* UART0 FIFO Control Register */
#define MA_U0FCR_UART0_MASK    0x000000C7  /* Used bits */
#define MA_U0LCR_UART0         0x00000003  /* UART0 Line Control Register */
#define MA_U0LCR_UART0_MASK    0x000000FF  /* Used bits */
#define MA_U0SCR_UART0         0x00000000  /* UART0 Scratch Pad Register */
#define MA_U0SCR_UART0_MASK    0x000000FF  /* Used bits */

/*
**===========================================================================
**  3.2     Internal macros
**===========================================================================
*/

#define U0IER_IEM              (U0IER_RXLSIE |\
                               U0IER_THREIE |\
                               U0IER_RDAIE)         /* Interrupt enable mask */

#define U0LSR_ERR              (U0LSR_RXFE  |\
                               U0LSR_BI    |\
                               U0LSR_FE    |\
                               U0LSR_PE    |\
                               U0LSR_OE)      /* LSR Register Error Mask */

/*
**===========================================================================
**  3.3     Internal type definitions
**===========================================================================
*/

typedef struct
{
    /*--- General ---*/
    U8  EOS;

} UARTDataType;

/*
**===========================================================================
**  3.4     Global variables (declared as 'extern' in some header file)
**===========================================================================
*/

/*
**===========================================================================
**  3.5     Internal function prototypes (defined in Section 5)
**===========================================================================
*/

/*
**===========================================================================
**  3.6     Internal variables
**===========================================================================
*/

static volatile UARTDataType UART0;


/*
**===========================================================================
**  4.      GLOBAL FUNCTIONS (declared as 'extern' in some header file)
**===========================================================================
*/



void MA_Init_UART0( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Initializes the UART0 module. Only sets those registers with 
**      values not equal to the power-on reset values.
**
**  Parameters:
**      None
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{

    /*--- Handle user code on function entry ---*/
    ENTER_MA_INIT_UART0;

    /*
    **----------------------------
    **  Initialize UART channel 0
    **----------------------------
    */
    /*--- Disable interrupts ---*/
    U0IER = 0;              

    /*--- Initialize registers ---*/
    U0LCR    = ( U0LCR & ~MA_U0LCR_UART0_MASK ) | MA_U0LCR_UART0;

    /*--- Initialize baudrate ---*/
    U0LCR_bit.DLAB = 1;    /* Open the divisor register access */    
    U0DLM = 0x0;
    U0DLL = 0x10;

    U0LCR_bit.DLAB = 0;    /* Close the divisor register access */

    /*--- Configure UART0 data block ---*/
    UART0.EOS = '\n';

    /*--- Handle user code on function exit ---*/
    EXIT_MA_INIT_UART0;

} /* MA_Init_UART0 */





void MA_Reset_UART0( void ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Resets the UART0 module. Sets all registers.
**
**  Parameters:
**      None
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{

    /*--- Handle user code on function entry ---*/
    ENTER_MA_RESET_UART0;

    /*
    **-----------------------
    **  Reset UART channel 0
    **-----------------------
    */
    /*--- Disable interrupts ---*/
    U0IER = 0;              

    /*--- Initialize registers ---*/
    U0LCR    = ( U0LCR & ~MA_U0LCR_UART0_MASK ) | MA_U0LCR_UART0;
    U0FCR    = ( U0FCR & ~MA_U0FCR_UART0_MASK ) | MA_U0FCR_UART0;

    /*--- Initialize baudrate ---*/
    U0LCR_bit.DLAB = 1;          /* Open the divisor register access */
    U0DLM = 0x0;
    U0DLL = 0x10;
    U0LCR_bit.DLAB = 0;          /* Close the divisor register access */

    /*--- Configure UART0 data block ---*/
    UART0.EOS = '\n';

    /*--- Handle user code on function exit ---*/
    EXIT_MA_RESET_UART0;

} /* MA_Reset_UART0 */





void MA_PutChar_UART0( U8 Data ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Send a character on the UART 0 channel. This function waits until
**      the transmit FIFO has space for the character to transmit.
**
**  Parameters:
**      Data        The character to transmit
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{

    /*--- Handle user code on function entry ---*/
    ENTER_MA_PUTCHAR_UART0;

    while(          U0LSR_bit.THRE           == U0LSR_THRE_VALID )
    {
        /*--- Wait for transmit holding register empty ---*/
        ;
    }

    /*--- Write character to transmit ---*/
    U0THR = Data;

    /*--- Handle user code on function exit ---*/
    EXIT_MA_PUTCHAR_UART0;
    
    return;

} /* MA_PutChar_UART0 */





void MA_PutString_UART0( C8 *pString ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Send a zero terminated string of characters to the UART 0 channel.
**      This function waits indefinitely until the string has been sent 
**      successfully.
**
**  Parameters:
**      pString     A pointer to the string to transmit
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{
    /*--- Handle user code on function entry ---*/
    ENTER_MA_PUTSTRING_UART0;
    
    /*--- Transmit data bytes ---*/
    while( *pString )
    {
        /*--- Transmit a data byte ---*/
        MA_PutChar_UART0( (U8)(*pString++) );
    }

    /*--- Handle user code on function exit ---*/
    EXIT_MA_PUTSTRING_UART0;
    
    return;

} /* MA_PutString_UART0 */





void MA_PutData_UART0( U8 *pData, U8 Length ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Send a number of characters to the UART 0 channel. 
**      This function waits indefinitely until the string has been sent 
**      successfully.
**
**  Parameters:
**      pData       A pointer to the data to transmit
**      Length      The number of bytes to write
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{
    /*--- Handle user code on function entry ---*/
    ENTER_MA_PUTDATA_UART0;
    
    /*--- Transmit data bytes ---*/
    while( Length-- )
    {
        /*--- Transmit a data byte ---*/
        MA_PutChar_UART0( *pData++ );
    }

    /*--- Handle user code on function exit ---*/
    EXIT_MA_PUTDATA_UART0;
    
    return;

} /* MA_PutData_UART0 */





S8 MA_GetChar_UART0( U8 *pChar ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Receive a character from the UART 0 channel. This function returns
**      immediately if a character cannot be received successfully.
**
**  Parameters:
**      pChar       A pointer to where the received character shall be saved.
**
**  Returns:
**      MA_OK       on success
**      MA_ERROR    if Break indicator, Framing, Parity or Overrun error 
**                  detected
**      MA_EMPTY    if the receive buffer is empty
**
**---------------------------------------------------------------------------
*/
{

    U8  Status;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_GETCHAR_UART0;
    
    Status = U0LSR;

    /*--- Any characters received? ---*/
    if( Status & U0LSR_DR_VALID )
    {
        /*--- Detect potential errors ---*/
        if(( Status & U0LSR_ERR ) != 0 )
        {
            /*--- Parity, framing, overrun or break indication detected ---*/
            *pChar = U0RBR;

            /*--- Handle user code on function exit ---*/
            EXIT_MA_GETCHAR_UART0;

            return MA_ERROR;
        }
        else
        {    
            /*--- Read the character ---*/
            *pChar = U0RBR;
        }
    }
    else
    {
        /*--- Handle user code on function exit ---*/
        EXIT_MA_GETCHAR_UART0;

        /*--- UART0 is empty ---*/
        return MA_EMPTY;
    }
    
    /*--- Handle user code on function exit ---*/
    EXIT_MA_GETCHAR_UART0;
    
    return MA_OK;

} /* MA_GetChar_UART0 */





S8 MA_GetString_UART0( C8 *pString , U8 Size ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Receive a string of characters from the UART 0 channel. Use the
**      function MA_SetEOS_UART0() to configure the end of string symbol,
**      if it is different from '\n'.
**
**      This function waits indefinitely until the string has been received 
**      successfully.
**	
**  Parameters:
**      pString     A pointer to the receive buffer
**      Size        The size of the receive buffer
**
**  Returns:
**      MA_OK       on success
**      MA_ERROR    if error detected
**      MA_FULL     if the receive buffer became full
**
**---------------------------------------------------------------------------
*/
{
    U8 Count;
    S8 Status;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_GETSTRING_UART0;
    
    /*--- Receive data bytes ---*/
    Count = 0;

    /*--- Wait for received string ---*/
    do
    {
        /*--- Wait for received character ---*/
        do 
        {
            Status = MA_GetChar_UART0( (U8 *)pString );
            if( Status == MA_ERROR )
            {
                /*--- Append zero termination ---*/
                *pString = '\0';

                /*--- Handle user code on function exit ---*/
                EXIT_MA_GETSTRING_UART0;
    
                /*--- Problem detected ---*/
                return MA_ERROR;
            }
        } while( Status != MA_OK );

    } while( ( *pString++ != UART0.EOS ) && ( ++Count < ( Size-1 ) ) );

    /*--- Append zero termination ---*/
    *pString = '\0';

    /*--- Determine return code ---*/
    if( ++Count == Size )
    {
        /*--- Handle user code on function exit ---*/
        EXIT_MA_GETSTRING_UART0;

        return MA_FULL;
    }
    /*--- Handle user code on function exit ---*/
    EXIT_MA_GETSTRING_UART0;

    return MA_OK;
        

} /* MA_GetString_UART0 */





S8 MA_GetData_UART0( U8 *pData, U8 Length ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Receive a number of characters from the UART 0 channel. 
**
**      This function waits indefinitely until the data has been received 
**      successfully.
**	
**  Parameters:
**      pData       A pointer to the receive buffer
**      Length      The number of bytes to read
**
**  Returns:
**      MA_OK       on success
**      MA_ERROR    if error detected
**
**---------------------------------------------------------------------------
*/
{
    S8 Status;

    /*--- Handle user code on function entry ---*/
    ENTER_MA_GETDATA_UART0;
    
    /*--- Wait for received data ---*/
    do
    {
        /*--- Wait for received character ---*/
        do 
        {
            Status = MA_GetChar_UART0( pData );
            if( Status == MA_ERROR )
            {
                /*--- Handle user code on function exit ---*/
                EXIT_MA_GETDATA_UART0;
    
                /*--- Problem detected ---*/
                return MA_ERROR;
            }
        } while( Status != MA_OK );

        *pData++;

    } while( --Length );

    /*--- Handle user code on function exit ---*/
    EXIT_MA_GETDATA_UART0;

    return MA_OK;
        

} /* MA_GetData_UART0 */





void MA_SetEOS_UART0( U8 Symbol ) 
/*
**---------------------------------------------------------------------------
**
**  Abstract:
**      Set the end of string symbol used by the MA_GetString_UART0() 
**      function.
**
**  Parameters:
**      Symbol	    End of string symbol
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
{
   /*--- Handle user code on function entry ---*/
    ENTER_MA_SETEOS_UART0;

    /*--- Set end of string symbol ---*/
    UART0.EOS = Symbol;

    /*--- Handle user code on function exit ---*/
    EXIT_MA_SETEOS_UART0;
      
} /* MA_SetEOS_UART0 */





/*
**===========================================================================
**  5.      INTERNAL FUNCTIONS (declared in Section 3.5)
**===========================================================================
*/

/*
**===========================================================================
** END OF FILE
**===========================================================================
*/ 



