// iostream -- ios::Init members, initialize standard streams
#include <yvals.h>

#if _DLIB_FILE_DESCRIPTOR

#include <fstream>

#else

#include <xxnofilebuf>

#endif

#include <istream>	/* NOT <iostream>, don't want Init object constructed */
#include <new>
#include "dtor_rec.h"  // To get __func_ptr
_STD_BEGIN

		// OBJECT DECLARATIONS
static int _Init_cnt = -1;	// net constructors - destructors

#if _DLIB_FILE_DESCRIPTOR
#pragma object_attribute = __no_init
static filebuf fin;             // uninitialized cin buffer
#pragma object_attribute = __no_init
static filebuf fout;            // uninitialized cout buffer
#else
#pragma object_attribute = __no_init
static __nofilebuf fin;         // uninitialized cin buffer
#pragma object_attribute = __no_init
static __nofilebuf fout;        // uninitialized cout buffer
#endif

// Hack to make initialization of standard streams run only if needed
static void FirstPassInit();
#ifdef _DIFUNCT_MEMORY
  #pragma type_attribute = _DIFUNCT_MEMORY
#endif /* _DIFUNCT_MEMORY */
#pragma location = "DIFUNCT"
static __func_ptr const InitStreams[2] = { 0, FirstPassInit };
// This is to ensure that __call_cctors is always called
extern ostream cout;
#pragma required = InitStreams
static char InitStreamsHelper = cout.fill();

#pragma required = InitStreamsHelper
#pragma object_attribute = __no_init
istream cin;                    // uninitialized cin
#pragma required = InitStreamsHelper
#pragma object_attribute = __no_init
ostream cout;                   // uninitialized cout

static void InitStandardStreams()
	{	// initialize standard streams first time
	bool doinit;
		{	// lock and test whether to initialize
		_Lockit lock(_LOCK_STREAM);
		if (0 <= _Init_cnt)
			++_Init_cnt, doinit = false;	// redundant initialization, skip
		else
			_Init_cnt = 1, doinit = true;	// first initialization, do it
		}

	if (doinit)
		{	// initialize standard streams
#if _DLIB_FILE_DESCRIPTOR
		new (&fin) filebuf(stdin);
		new (&fout) filebuf(stdout);
#else
		new (&fin) __nofilebuf();
		new (&fout) __nofilebuf();
#endif
		new (&cin) istream(&fin, true);
		new (&cout) ostream(&fout, true);
		cin.tie(&cout);
		}
        }

static void FlushStandardStreams()
	{	// flush standard streams on last destruction
	bool doflush;
		{	// lock and test whether to flush streams
		_Lockit lock(_LOCK_STREAM);
		if (--_Init_cnt == 0)
			doflush = true;	// last destruction, do it
		else
			doflush = false;	// not last destruction, skip
		}

	if (doflush)
		{	// flush standard streams, but don't destroy
		cout.flush();
		}
	}

static void FirstPassInit()
{
  InitStandardStreams();
  // Ensure that flush is run last.
  struct X { ~X() { FlushStandardStreams(); } };
  static X x;
}

// These must exist, if the user explicitly creates an object of this type,
// and so that linking with code compiled with the old headers still works.
ios_base::Init::Init()
	{	// initialize standard streams first time
          InitStandardStreams();
	}

ios_base::Init::~Init()
	{	// flush standard streams on last destruction
          FlushStandardStreams();
	}
_STD_END

char _PJP_CPP_Copyright[] =
	"Copyright (c) 1992-2002 by P.J. Plauger,"
	" licensed by Dinkumware, Ltd."
	" ALL RIGHTS RESERVED.";

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
