/* setlocale.c Forwardning functions for all locales. */
/* Copyright (C) 2003 IAR Systems.  All rights reserved. */
#include <yvals.h>

#include <locale.h>
#include <string.h>

#include <xtls.h>
#include <xmtloc.h>
#include <xtime.h>
#include <xencoding.h>


/* Writeable copy of the _Tzone and _Isdst fields of _Times. */
_TLS_DEFINE_INIT(   ,char const *, _TzoneStr) = "";
_TLS_DEFINE_NO_INIT(,char const *, _TzoneStr);

_TLS_DEFINE_INIT(   ,char const *, _IsdstStr) = "";
_TLS_DEFINE_NO_INIT(,char const *, _IsdstStr);


#if _DLIB_FULL_LOCALE_SUPPORT

/*
 * The current selected locales.  This is an array of id:s for each
 * locale category.
 */

struct _Locale_Profile_t
{
  int _Array[_NCAT];
  int _Encoding;
};


_TLS_DEFINE_INIT(, struct _Locale_Profile_t, _Locale_Profile) =
{
  /* By default, the "C" locale is selected for all categories. */
  {
    _LocaleC_id, _LocaleC_id, _LocaleC_id, _LocaleC_id,
    _LocaleC_id, _LocaleC_id
  },
  _EncodingSb_Id
};
_TLS_DEFINE_NO_INIT(, struct _Locale_Profile_t, _Locale_Profile);


/*
 * The lconv array.
 */

#define _LOCALE_LIST_HELPER(x) extern const struct lconv _Locale##x##_lconv;
_LOCALE_LIST
#undef _LOCALE_LIST_HELPER


const struct lconv * _Locale_LconvArray[_LocaleCount] =
{
#define _LOCALE_LIST_HELPER(n) &_Locale##n##_lconv,
  _LOCALE_LIST
#undef _LOCALE_LIST_HELPER
};


/*
 * Locale descriptor (only the name, so far).
 */

#define _LOCALE_LIST_HELPER(x)                                          \
extern const char _Locale##x##_Name[];

_LOCALE_LIST

#undef _LOCALE_LIST_HELPER

struct _LocaleDesc
{
  char const * _Name;
};

#define _LOCALE_LIST_HELPER(n) { _Locale##n##_Name },

const struct _LocaleDesc _LocaleDescArray[] =
{
  _LOCALE_LIST
};

#undef _LOCALE_LIST_HELPER


/*
 * Encodings.
 */

char const * const _EncodingNameArray[] =
{
  "Sb",
#ifdef _ENCODING_USE_UTF8
  "Utf8"
#endif
};


/*
 * Find Id from name.
 */

/* Note: The name may be followed by a ";" or "." (this function is
 * used by _LocaleParseString). */
static int _LocaleGetId(char const * name)
{
  /* Find the part of "name" that should be compared. */
  char const * endPtr = strpbrk(name, ";.");
  size_t len;
  int i;

  if (endPtr == 0)
  {
    len = strlen(name);
  }
  else
  {
    len = endPtr - name;
  }

  for (i = 0; i < sizeof(_LocaleDescArray)/sizeof(_LocaleDescArray[0]); ++i)
  {
    if (strncmp(name, _LocaleDescArray[i]._Name, len) == 0)
    {
      return i;
    }
  }

  return 0;
}


/*
 * Get locale for a specific category.
 */

int _LocaleForCat(int cat)
{
  return _TLS_DATA_PTR(_Locale_Profile)->_Array[cat];
}


int _LocaleEncoding(void)
{
  return _TLS_DATA_PTR(_Locale_Profile)->_Encoding;
}


/*
 * Encoding support.
 */


static int _EncodingGetId(char const * name)
{
  int i;

  for (i = 0; i < _EncodingCount; ++i)
  {
    if (strcmp(name, _EncodingNameArray[i]) == 0)
    {
      break;
    }
  }

  return i;
}



/*
 * setlocale and help functions.
 */

/* _LocaleParseString - Parse a string on the forms:
 *    locale[.encoding]
 *    locale;locale;locale;locale;locale;locale;locale[.encoding]
 */

static void _LocaleParseString(int cat,
                               char const * locname,
                               struct _Locale_Profile_t * profile)
{
  char const * endPtr;

  endPtr = strpbrk(locname, ";.");

  if (   endPtr == 0
      || *endPtr == '.')
  {
    /* Only one name, the full name of the locale. */

    int id = _LocaleGetId(locname);
    int i;

    for (i = 0; i < _NCAT; ++i)
    {
      if (_CATMASK(i) & cat)
      {
        profile->_Array[i] = id;
      }
    }
  }
  else
  {
    /* One name per locale category. */
    int i;

    for (i = 0; i < _NCAT; ++i)
    {
      if (_CATMASK(i) & cat)
      {
        profile->_Array[i] = _LocaleGetId(locname);
      }

      if (*endPtr != ';')
      {
        break;
      }
      else
      {
        ++endPtr;                 /* Step past ";" */
        locname = endPtr;

        endPtr = strpbrk(endPtr, ";.");
      }
    }
  }

  if (*endPtr == '.')
  {
    profile->_Encoding = _EncodingGetId(endPtr + 1);
  }
}


static void _LocaleMakeString(char * s)
{
  /*
   * Build a string that represents the current locale.
   */

  int allSame = 1;

  int i;
  for (i = 0; i < _NCAT; ++i)
  {
    if (   _TLS_DATA_PTR(_Locale_Profile)->_Array[i]
        != _TLS_DATA_PTR(_Locale_Profile)->_Array[0])
    {
      allSame = 0;
      break;
    }
  }

  if (allSame)
  {
    /* Just write name of one locale. */
    strcpy(s, _LocaleDescArray[_TLS_DATA_PTR(_Locale_Profile)->_Array[0]]._Name);
  }
  else
  {
    *s = '\0';

    for (i = 0; i < _NCAT; ++i)
    {
      if (i != 0)
      {
        strcat(s, ";");
      }
      strcat(s, _LocaleDescArray[_TLS_DATA_PTR(_Locale_Profile)->_Array[i]]._Name);
    }
  }

  if (_LocaleEncoding() != _EncodingSb_Id)
  {
    /* Append ".Enc" to the string. */
    strcat(s, ".");
    strcat(s, _EncodingNameArray[_TLS_DATA_PTR(_Locale_Profile)->_Encoding]);
  }
}


static void _LocaleUpdate(struct _Locale_Profile_t const * profile)
{
  int i;

  for (i = 0; i < _NCAT; ++i)
  {
    if (profile->_Array[i] != _TLS_DATA_PTR(_Locale_Profile)->_Array[i])
    {
      struct lconv const * p = _Locale_LconvArray[profile->_Array[i]];

      switch (i)
      {
      case _LC_MONETARY:
        {       /* set monetary category */
          struct lconv *plconv = _TLS_DATA_PTR(_Locale_lconv);

          plconv->currency_symbol    = p->currency_symbol;
          plconv->int_curr_symbol    = p->int_curr_symbol;
          plconv->mon_decimal_point  = p->mon_decimal_point;
          plconv->mon_grouping       = p->mon_grouping;
          plconv->mon_thousands_sep  = p->mon_thousands_sep;
          plconv->negative_sign      = p->negative_sign;
          plconv->positive_sign      = p->positive_sign;

          plconv->frac_digits        = p->frac_digits;
          plconv->n_cs_precedes      = p->n_cs_precedes;
          plconv->n_sep_by_space     = p->n_sep_by_space;
          plconv->n_sign_posn        = p->n_sign_posn;
          plconv->p_cs_precedes      = p->p_cs_precedes;
          plconv->p_sep_by_space     = p->p_sep_by_space;
          plconv->p_sign_posn        = p->p_sign_posn;

          plconv->int_frac_digits    = p->int_frac_digits;
        }
        break;

      case _LC_NUMERIC:
        {       /* set numeric category */
          struct lconv *plconv = _TLS_DATA_PTR(_Locale_lconv);

          plconv->decimal_point  = p->decimal_point[0] != '\0'
            ? p->decimal_point : (char *)".";
          plconv->grouping       = p->grouping;
          plconv->thousands_sep  = p->thousands_sep;
          plconv->_Frac_grouping = p->_Frac_grouping;
          plconv->_Frac_sep      = p->_Frac_sep;
          plconv->_False         = p->_False;
          plconv->_True          = p->_True;
        }
        break;

      case _LC_TIME:
        {
          _Tinfo const * ptimes = _GetptimesFor(profile->_Array[i]);
          *_TLS_DATA_PTR(_TzoneStr) = ptimes->_Tzone;
          *_TLS_DATA_PTR(_IsdstStr) = ptimes->_Isdst;
        }
        break;

      case _LC_MESSAGE:
        {       /* set message category */
          struct lconv *plconv = _TLS_DATA_PTR(_Locale_lconv);

          plconv->_No  = p->_No;
          plconv->_Yes = p->_Yes;
        }
        break;
      }
    }
  }
}


_TLS_ARR_DEF(static, char, _LocaleReturnString, 80);

/* set new locale */
char *(setlocale)(int cat, char const *locname)
{
  struct _Locale_Profile_t temp_profile = *_TLS_DATA_PTR(_Locale_Profile);

  /*
   * Set the new locale, as described in the string "locname".
   */

  _LocaleParseString(cat, locname, & temp_profile);


  /*
   * Create a string representing the current locale (remember, the
   * string might use the same physical memory as locname).
   */

  _LocaleMakeString(_TLS_ARR(_LocaleReturnString));


  /*
   * Update the current locale and make sure the lconv structure
   * also is updated.
   */

  _LocaleUpdate(& temp_profile);


  /*
   * Store the new profile.
   */

  *_TLS_DATA_PTR(_Locale_Profile) = temp_profile;

  return _TLS_ARR(_LocaleReturnString);
}

#endif /* _DLIB_FULL_LOCALE_SUPPORT */
