// string -- template string support functions
#include <istream>
_STD_BEGIN

// _String_base

#pragma diag_suppress = Pe236, Pe340

void _String_base::_Xlen() const
	{	// report a length_error
	_THROW(length_error, "string too long");
	}

void _String_base::_Xran() const
	{	// report an out_of_range error
	_THROW(out_of_range, "invalid string position");
	}

#pragma diag_default = Pe236, Pe340

// string

const string::size_type string::npos =
	(string::size_type)(-1);	// define npos as largest positive value

string::_Myt& string::append(const _Myt& _Right, size_type _Roff,
                             size_type _Count)
{     // append _Right [_Roff, _Roff + _Count)
  if (_Right.size() < _Roff)
    _String_base::_Xran();    // _Roff off end
  size_type _Num = _Right.size() - _Roff;
  if (_Num < _Count)
    _Count = _Num;    // trim _Count to size
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long

  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and append new stuff
    _Traits::copy(_Myptr() + _Mysize,
                  _Right._Myptr() + _Roff, _Count);
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::append(const _Elem *_Ptr, size_type _Count)
{     // append [_Ptr, _Ptr + _Count)
  if (_Inside(_Ptr))
    return (append(*this, _Ptr - _Myptr(), _Count));  // substring
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long

  size_type _Num;
  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and append new stuff
    _Traits::copy(_Myptr() + _Mysize, _Ptr, _Count);
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::append(size_type _Count, _Elem _Ch)
{     // append _Count * _Ch
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long

  size_type _Num;
  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and append new stuff using assign
    _Traits::assign(_Myptr() + _Mysize, _Count, _Ch);
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::assign(const _Myt& _Right, size_type _Roff,
                             size_type _Count)
{     // assign _Right [_Roff, _Roff + _Count)
  if (_Right.size() < _Roff)
    _String_base::_Xran();    // _Roff off end
  size_type _Num = _Right.size() - _Roff;
  if (_Count < _Num)
    _Num = _Count;    // trim _Num to size

  if (this == &_Right)
    erase((size_type)(_Roff + _Num)), erase(0, _Roff);        // substring
  else if (_Grow(_Num))
  {   // make room and assign new stuff
    _Traits::copy(_Myptr(), _Right._Myptr() + _Roff, _Num);
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::assign(const _Elem *_Ptr, size_type _Num)
{     // assign [_Ptr, _Ptr + _Num)
  if (_Inside(_Ptr))
    return (assign(*this, _Ptr - _Myptr(), _Num));    // substring

  if (_Grow(_Num))
  {   // make room and assign new stuff
    _Traits::copy(_Myptr(), _Ptr, _Num);
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::assign(size_type _Count, _Elem _Ch)
{     // assign _Count * _Ch
  if (_Count == npos)
    _String_base::_Xlen();    // result too long

  if (_Grow(_Count))
  {   // make room and assign new stuff
    _Traits::assign(_Myptr(), _Count, _Ch);
    _Eos(_Count);
  }
  return (*this);
}

string::_Myt& string::insert(size_type _Off, const _Myt& _Right,
                             size_type _Roff, size_type _Count)
{     // insert _Right [_Roff, _Roff + _Count) at _Off
  if (_Mysize < _Off || _Right.size() < _Roff)
    _String_base::_Xran();    // _Off or _Roff off end
  size_type _Num = _Right.size() - _Roff;
  if (_Num < _Count)
    _Count = _Num;    // trim _Count to size
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long

  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and insert new stuff
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off, _Mysize - _Off);   // empty out hole
    if (this == &_Right)
      _Traits::move(_Myptr() + _Off,
                    _Myptr() + (_Off < _Roff ? _Roff + _Count : _Roff),
                    _Count);  // substring
    else
      _Traits::copy(_Myptr() + _Off,
                    _Right._Myptr() + _Roff, _Count); // fill hole
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::insert(size_type _Off, const _Elem *_Ptr,
                             size_type _Count)
{     // insert [_Ptr, _Ptr + _Count) at _Off
  if (_Inside(_Ptr))
    return (insert(_Off, *this,
                   _Ptr - _Myptr(), _Count)); // substring
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long
  size_type _Num;
  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and insert new stuff
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off, _Mysize - _Off);   // empty out hole
    _Traits::copy(_Myptr() + _Off, _Ptr, _Count);     // fill hole
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::insert(size_type _Off, size_type _Count, _Elem _Ch)
{     // insert _Count * _Ch at _Off
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (npos - _Mysize <= _Count)
    _String_base::_Xlen();    // result too long
  size_type _Num;
  if (0 < _Count && _Grow(_Num = _Mysize + _Count))
  {   // make room and insert new stuff
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off, _Mysize - _Off);   // empty out hole
    _Traits::assign(_Myptr() + _Off, _Count, _Ch);    // fill hole
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::erase(size_type _Off, size_type _Count)
{     // erase elements [_Off, _Off + _Count)
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (_Mysize - _Off < _Count)
    _Count = _Mysize - _Off;  // trim _Count
  if (0 < _Count)
  {   // move elements down
    _Traits::move(_Myptr() + _Off, _Myptr() + _Off + _Count,
                  _Mysize - _Off - _Count);
    size_type _Newsize = _Mysize - _Count;
    _Eos(_Newsize);
  }
  return (*this);
}

string::_Myt& string::replace(size_type _Off, size_type _N0, const _Myt& _Right,
                              size_type _Roff, size_type _Count)
{     // replace [_Off, _Off + _N0) with _Right [_Roff, _Roff + _Count)
  if (_Mysize < _Off || _Right.size() < _Roff)
    _String_base::_Xran();    // _Off or _Roff off end
  if (_Mysize - _Off < _N0)
    _N0 = _Mysize - _Off;     // trim _N0 to size
  size_type _Num = _Right.size() - _Roff;
  if (_Num < _Count)
    _Count = _Num;    // trim _Count to size
  if (npos - _Count <= _Mysize - _N0)
    _String_base::_Xlen();    // result too long

  size_type _Nm = _Mysize - _N0 - _Off;       // length of preserved tail
  size_type _Newsize = _Mysize + _Count - _N0;
  if (_Mysize < _Newsize)
    _Grow(_Newsize);

  if (this != &_Right)
  {   // no overlap, just move down and copy in new stuff
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);        // empty hole
    _Traits::copy(_Myptr() + _Off,
                  _Right._Myptr() + _Roff, _Count);   // fill hole
  }
  else if (_Count <= _N0)
  {   // hole doesn't get larger, just copy in substring
    _Traits::move(_Myptr() + _Off,
                  _Myptr() + _Roff, _Count);  // fill hole
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);        // move tail down
  }
  else if (_Roff <= _Off)
  {   // hole gets larger, substring begins before hole
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);        // move tail down
    _Traits::move(_Myptr() + _Off,
                  _Myptr() + _Roff, _Count);  // fill hole
  }
  else if (_Off + _N0 <= _Roff)
  {   // hole gets larger, substring begins after hole
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);        // move tail down
    _Traits::move(_Myptr() + _Off,
                  _Myptr() + (_Roff + _Count - _N0), _Count); // fill hole
  }
  else
  {   // hole gets larger, substring begins in hole
    _Traits::move(_Myptr() + _Off,
                  _Myptr() + _Roff, _N0);     // fill old hole
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);        // move tail down
    _Traits::move(_Myptr() + _Off + _N0, _Myptr() + _Roff + _Count,
                  _Count - _N0);      // fill rest of new hole
  }

  _Eos(_Newsize);
  return (*this);
}

string::_Myt& string::replace(size_type _Off, size_type _N0, const _Elem *_Ptr,
                              size_type _Count)
{     // replace [_Off, _Off + _N0) with [_Ptr, _Ptr + _Count)
  if (_Inside(_Ptr))
    return (replace(_Off, _N0, *this,
                    _Ptr - _Myptr(), _Count)); // substring, replace carefully
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (_Mysize - _Off < _N0)
    _N0 = _Mysize - _Off;     // trim _N0 to size
  if (npos - _Count <= _Mysize - _N0)
    _String_base::_Xlen();    // result too long
  size_type _Nm = _Mysize - _N0 - _Off;

  if (_Count < _N0)
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm); // smaller hole, move tail up
  size_type _Num;
  if ((0 < _Count || 0 < _N0) && _Grow(_Num = _Mysize + _Count - _N0))
  {   // make room and rearrange
    if (_N0 < _Count)
      _Traits::move(_Myptr() + _Off + _Count,
                    _Myptr() + _Off + _N0, _Nm);      // move tail down
    _Traits::copy(_Myptr() + _Off, _Ptr, _Count);     // fill hole
    _Eos(_Num);
  }
  return (*this);
}

string::_Myt& string::replace(size_type _Off, size_type _N0,
                              size_type _Count, _Elem _Ch)
{     // replace [_Off, _Off + _N0) with _Count * _Ch
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (_Mysize - _Off < _N0)
    _N0 = _Mysize - _Off;     // trim _N0 to size
  if (npos - _Count <= _Mysize - _N0)
    _String_base::_Xlen();    // result too long
  size_type _Nm = _Mysize - _N0 - _Off;

  if (_Count < _N0)
    _Traits::move(_Myptr() + _Off + _Count,
                  _Myptr() + _Off + _N0, _Nm);    // smaller hole, move tail up
  size_type _Num;
  if ((0 < _Count || 0 < _N0) && _Grow(_Num = _Mysize + _Count - _N0))
  {   // make room and rearrange
    if (_N0 < _Count)
      _Traits::move(_Myptr() + _Off + _Count,
                    _Myptr() + _Off + _N0, _Nm);      // move tail down
    _Traits::assign(_Myptr() + _Off, _Count, _Ch);    // fill hole
    _Eos(_Num);
  }
  return (*this);
}

string::size_type string::copy(_Elem *_Ptr, size_type _Count,
                               size_type _Off) const
{     // copy [_Off, _Off + _Count) to [_Ptr, _Ptr + _Count)
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (_Mysize - _Off < _Count)
    _Count = _Mysize - _Off;
  _Traits::copy(_Ptr, _Myptr() + _Off, _Count);
  return (_Count);
}

void string::swap(_Myt& _Right)
{     // exchange contents with _Right
  if (_Mybase::_Alval == _Right._Alval)
  {   // same allocator, swap control information
    _Bxty _Tbx = _Bx;
    _Bx = _Right._Bx, _Right._Bx = _Tbx;

    size_type _Tlen = _Mysize;
    _Mysize = _Right._Mysize, _Right._Mysize = _Tlen;

    size_type _Tres = _Myres;
    _Myres = _Right._Myres, _Right._Myres = _Tres;
  }
  else
  {   // different allocator, do multiple assigns
    _Myt _Tmp = *this; *this = _Right, _Right = _Tmp;
  }
}

string::size_type string::find(const _Elem *_Ptr, size_type _Off,
                               size_type _Count) const
{     // look for [_Ptr, _Ptr + _Count) beginnng at or after _Off
  if (_Count == 0 && _Off <= _Mysize)
    return (_Off);    // null string always matches (if inside string)

  size_type _Nm;
  if (_Off < _Mysize && _Count <= (_Nm = _Mysize - _Off))
  {   // room for match, look for it
    const _Elem *_Uptr, *_Vptr;
    for (_Nm -= _Count - 1, _Vptr = _Myptr() + _Off;
         (_Uptr = _Traits::find(_Vptr, _Nm, *_Ptr)) != 0;
         _Nm -= _Uptr - _Vptr + 1, _Vptr = _Uptr + 1)
      if (_Traits::compare(_Uptr, _Ptr, _Count) == 0)
        return (_Uptr - _Myptr());    // found a match
  }

  return (npos);      // no match
}

string::size_type string::rfind(const _Elem *_Ptr, size_type _Off,
                                size_type _Count) const
{     // look for [_Ptr, _Ptr + _Count) beginning before _Off
  if (_Count == 0)
    return (_Off < _Mysize ? _Off : _Mysize); // null always matches
  if (_Count <= _Mysize)
  {   // room for match, look for it
    const _Elem *_Uptr = _Myptr() +
      (_Off < _Mysize - _Count ? _Off : _Mysize - _Count);
    for (; ; --_Uptr)
      if (_Traits::eq(*_Uptr, *_Ptr)
          && _Traits::compare(_Uptr, _Ptr, _Count) == 0)
        return (_Uptr - _Myptr());    // found a match
      else if (_Uptr == _Myptr())
        break;        // at beginning, no more chance for match
  }

  return (npos);      // no match
}

string::size_type string::find_first_of(const _Elem *_Ptr, size_type _Off,
                                        size_type _Count) const
{     // look for one of [_Ptr, _Ptr + _Count) at or after _Off
  if (0 < _Count && _Off < _Mysize)
  {   // room for match, look for it
    const _Elem *const _Vptr = _Myptr() + _Mysize;
    for (const _Elem *_Uptr = _Myptr() + _Off; _Uptr < _Vptr; ++_Uptr)
      if (_Traits::find(_Ptr, _Count, *_Uptr) != 0)
        return (_Uptr - _Myptr());    // found a match
  }

  return (npos);      // no match
}

string::size_type string::find_last_of(const _Elem *_Ptr, size_type _Off,
                                       size_type _Count) const
{     // look for one of [_Ptr, _Ptr + _Count) before _Off
  if (0 < _Count && 0 < _Mysize)
    for (const _Elem *_Uptr = _Myptr()
           + (_Off < _Mysize ? _Off : _Mysize - 1); ; --_Uptr)
      if (_Traits::find(_Ptr, _Count, *_Uptr) != 0)
        return (_Uptr - _Myptr());    // found a match
      else if (_Uptr == _Myptr())
        break;        // at beginning, no more chance for match

  return (npos);      // no match
}

string::size_type string::find_first_not_of(const _Elem *_Ptr, size_type _Off,
                                            size_type _Count) const
{     // look for none of [_Ptr, _Ptr + _Count) at or after _Off
  if (_Off < _Mysize)
  {   // room for match, look for it
    const _Elem *const _Vptr = _Myptr() + _Mysize;
    for (const _Elem *_Uptr = _Myptr() + _Off; _Uptr < _Vptr; ++_Uptr)
      if (_Traits::find(_Ptr, _Count, *_Uptr) == 0)
        return (_Uptr - _Myptr());
  }
  return (npos);
}

string::size_type string::find_last_not_of(const _Elem *_Ptr, size_type _Off,
                                           size_type _Count) const
{     // look for none of [_Ptr, _Ptr + _Count) before _Off
  if (0 < _Mysize)
    for (const _Elem *_Uptr = _Myptr()
           + (_Off < _Mysize ? _Off : _Mysize - 1); ; --_Uptr)
      if (_Traits::find(_Ptr, _Count, *_Uptr) == 0)
        return (_Uptr - _Myptr());
      else if (_Uptr == _Myptr())
        break;
  return (npos);
}

int string::compare(size_type _Off, size_type _N0, const _Myt& _Right,
              size_type _Roff, size_type _Count) const
{     // compare [_Off, _Off + _N0) with _Right [_Roff, _Roff + _Count)
  if (_Right.size() < _Roff)
    _String_base::_Xran();    // _Off off end
  if (_Right._Mysize - _Roff < _Count)
    _Count = _Right._Mysize - _Roff;  // trim _Count to size
  return (compare(_Off, _N0, _Right._Myptr() + _Roff, _Count));
}

int string::compare(size_type _Off, size_type _N0, const _Elem *_Ptr,
              size_type _Count) const
{     // compare [_Off, _Off + _N0) with [_Ptr, _Ptr + _Count)
  if (_Mysize < _Off)
    _String_base::_Xran();    // _Off off end
  if (_Mysize - _Off < _N0)
    _N0 = _Mysize - _Off;     // trim _N0 to size

  size_type _Ans = _N0 == 0 ? 0
    : _Traits::compare(_Myptr() + _Off, _Ptr,
                       _N0 < _Count ? _N0 : _Count);
  return (_Ans != 0 ? (int)_Ans : _N0 < _Count ? -1
          : _N0 == _Count ? 0 : +1);
}

#pragma diag_suppress = Pe236

void string::_Copy(size_type _Newsize, size_type _Oldlen)
{     // copy _Oldlen elements to newly allocated buffer
  size_type _Newres = _Newsize | _ALLOC_MASK;
  if (max_size() < _Newres)
    _Newres = _Newsize;       // undo roundup if too big
  else if (_Newres / 3 < _Myres / 2
           && _Myres <= max_size() - _Myres / 2)
    _Newres = _Myres + _Myres / 2;    // grow exponentially if possible
  _Elem *_Ptr;

  _TRY_BEGIN
    _Ptr = _Mybase::_Alval.allocate(_Newres + 1);
  _CATCH_ALL
    _Newres = _Newsize;       // allocation failed, undo roundup and retry
  _TRY_BEGIN
    _Ptr = _Mybase::_Alval.allocate(_Newres + 1);
  _CATCH_ALL
    _Tidy(true);      // failed again, discard storage and reraise
  _RERAISE;
  _CATCH_END
    _CATCH_END

    if (0 < _Oldlen)
      _Traits::copy(_Ptr, _Myptr(), _Oldlen); // copy existing elements
  _Tidy(true);
  _Bx._Ptr = _Ptr;
  _Myres = _Newres;
  _Eos(_Oldlen);
}

#pragma diag_default = Pe236

bool string::_Grow(size_type _Newsize, bool _Trim)
{     // ensure buffer is big enough, trim to size if _Trim is true
  if (max_size() < _Newsize)
    _String_base::_Xlen();    // result too long
  if (_Myres < _Newsize)
    _Copy(_Newsize, _Mysize); // reallocate to grow
  else if (_Trim && _Newsize < _BUF_SIZE)
    _Tidy(true,       // copy and deallocate if trimming to small string
          _Newsize < _Mysize ? _Newsize : _Mysize);
  else if (_Newsize == 0)
    _Eos(0);  // new size is zero, just null terminate
  return (0 < _Newsize);      // return true only if more work to do
}

void string::_Tidy(bool _Built, size_type _Newsize)
{     // initialize buffer, deallocating any storage
  if (!_Built)
    ;
  else if (_BUF_SIZE <= _Myres)
  {   // copy any leftovers to small buffer and deallocate
    _Elem *_Ptr = _Bx._Ptr;
    if (0 < _Newsize)
      _Traits::copy(_Bx._Buf, _Ptr, _Newsize);
    _Mybase::_Alval.deallocate(_Ptr, _Myres + 1);
  }
  _Myres = _BUF_SIZE - 1;
  _Eos(_Newsize);
}

// Top-level

#pragma diag_suppress = Pe340

istream& operator>>(istream& _Istr, string& _Str)
{       // extract a string
  typedef char_traits _Traits;
  typedef char_allocator _Alloc;
  typedef ctype _Ctype;
  typedef istream _Myis;
  typedef string _Mystr;
  typedef size_t _Mysizt;

  ios_base::iostate _State = ios_base::goodbit;
  bool _Changed = false;
  const _Myis::sentry _Ok(_Istr);

  if (_Ok)
  {     // state okay, extract characters
    const _Ctype& _Ctype_fac = _USE(_Istr.getloc(), _Ctype);
    _Str.erase();

    _TRY_IO_BEGIN
    _Mysizt _Size = 0 < _Istr.width()
      && (_Mysizt)_Istr.width() < _Str.max_size()
      ? (_Mysizt)_Istr.width() : _Str.max_size();
    _Traits::int_type _Meta = _Istr.rdbuf()->sgetc();

    for (; 0 < _Size; --_Size, _Meta = _Istr.rdbuf()->snextc())
      if(_Traits::eq_int_type(_Traits::eof(), _Meta))
      { // end of file, quit
        _State |= ios_base::eofbit;
        break;
      }
      else if (_Ctype_fac.is(_Ctype::space,
                             _Traits::to_char_type(_Meta)))
        break;  // whitespace, quit
      else
      { // add character to string
        _Str.append(1, _Traits::to_char_type(_Meta));
        _Changed = true;
      }
    _CATCH_IO_(_Istr)
  }

  _Istr.width(0);
  if (!_Changed)
    _State |= ios_base::failbit;
  _Istr.setstate(_State);
  return (_Istr);
}

istream& getline(istream& _Istr, string& _Str,
                 const char _Delim)
{       // get characters into string, discard delimiter
  typedef char_traits _Traits;
  typedef istream _Myis;
  ios_base::iostate _State = ios_base::goodbit;
  bool _Changed = false;
  const _Myis::sentry _Ok(_Istr, true);

  if (_Ok)
  {     // state okay, extract characters
    _TRY_IO_BEGIN
    _Str.erase();
    const _Traits::int_type _Metadelim =
      _Traits::to_int_type(_Delim);
    _Traits::int_type _Meta = _Istr.rdbuf()->sgetc();

    for (; ; _Meta = _Istr.rdbuf()->snextc())
      if (_Traits::eq_int_type(_Traits::eof(), _Meta))
      { // end of file, quit
        _State |= ios_base::eofbit;
        break;
      }
      else if (_Traits::eq_int_type(_Meta, _Metadelim))
      { // got a delimiter, discard it and quit
        _Changed = true;
        _Istr.rdbuf()->sbumpc();
        break;
      }
      else if (_Str.max_size() <= _Str.size())
      { // string too large, quit
        _State |= ios_base::failbit;
        break;
      }
      else
      { // got a character, add it to string
        _Str += _Traits::to_char_type(_Meta);
        _Changed = true;
      }
    _CATCH_IO_(_Istr)
  }

  if (!_Changed)
    _State |= ios_base::failbit;
  _Istr.setstate(_State);
  return (_Istr);
}

ostream& operator<<(ostream& _Ostr, const string& _Str)
{       // insert a string
  typedef char_traits _Traits;
  typedef char_allocator _Alloc;
  typedef ostream _Myos;
  typedef string _Mystr;
  typedef size_t _Mysizt;

  ios_base::iostate _State = ios_base::goodbit;
  _Mysizt _Size = _Str.size();
  _Mysizt _Pad = _Ostr.width() <= 0 || (_Mysizt)_Ostr.width() <= _Size
    ? 0 : (_Mysizt)_Ostr.width() - _Size;
  const _Myos::sentry _Ok(_Ostr);

  if (!_Ok)
    _State |= ios_base::badbit;
  else
  {     // state okay, insert characters
    _TRY_IO_BEGIN
    if ((_Ostr.flags() & ios_base::adjustfield) != ios_base::left)
      for (; 0 < _Pad; --_Pad)  // pad on left
        if (_Traits::eq_int_type(_Traits::eof(),
                                 _Ostr.rdbuf()->sputc(_Ostr.fill())))
        {       // insertion failed, quit
          _State |= ios_base::badbit;
          break;
        }

    if (_State == ios_base::goodbit)
      for (_Mysizt _Count = 0; _Count < _Size; ++_Count)
        if (_Traits::eq_int_type(_Traits::eof(),
                                 _Ostr.rdbuf()->sputc(_Str[_Count])))
        {       // insertion failed, quit
          _State |= ios_base::badbit;
          break;
        }

    if (_State == ios_base::goodbit)
      for (; 0 < _Pad; --_Pad)  // pad on right
        if (_Traits::eq_int_type(_Traits::eof(),
                                 _Ostr.rdbuf()->sputc(_Ostr.fill())))
        {       // insertion failed, quit
          _State |= ios_base::badbit;
          break;
        }
    _Ostr.width(0);
    _CATCH_IO_(_Ostr)
  }

  _Ostr.setstate(_State);
  return (_Ostr);
}

_STD_END

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
