/* values used by <float.h> macros -- IEEE 754 version */
#include <float.h>
#include "xmath.h"
_STD_BEGIN

		/* macros */
#define LOG2	0.30103
#define ONES	0xffff

#define FFRAC	(_FMANTISSA + 1)
#define FMAXE	_FMAX

#define DFRAC	(_DMANTISSA + 1)
#define DMAXE	_DMAX

#define LFRAC   (_LMANTISSA + 1)
#define LMAXE   _LMAX

#ifndef __IAR_SYSTEMS_ICC__
 #if _D0 == 0
  #define FINIT(w0, wx)	{w0, wx, 0, 0, 0, 0, 0, 0}	/* big-endian */
  #define DINIT(w0, wx)	{w0, wx, wx, wx, 0, 0, 0, 0}
  #define LINIT(w0, w1, wx)	{w0, w1, wx, wx, wx, wx, wx, wx}

 #elif _DLONG == 0
  #define FINIT(w0, wx)	{wx, w0, 0, 0, 0, 0}	/* little-endian, 64-bit */
  #define DINIT(w0, wx)	{wx, wx, wx, w0, 0, 0}
  #define LINIT(w0, w1, wx)	xxx	/* should never be used */

 #elif _DLONG == 1
  #define FINIT(w0, wx)	{wx, w0, 0, 0, 0, 0}	/* little-endian, 80-bit */
  #define DINIT(w0, wx)	{wx, wx, wx, w0, 0, 0}
  #define LINIT(w0, w1, wx)	{wx, wx, wx, w1, w0, 0}

 #else /* 1 < _DLONG */
  #define FINIT(w0, wx)	{wx, w0, 0, 0, 0, 0}	/* little-endian, 128-bit */
  #define DINIT(w0, wx)	{wx, wx, wx, w0, 0, 0}
  #define LINIT(w0, w1, wx)	{wx, wx, wx, wx, wx, wx, w1, w0}
 #endif
#endif /* __IAR_SYSTEMS_ICC__ */

		/* static data */
#ifndef _FLOAT_IS_DOUBLE
  #if __FLOAT_SIZE__ != 4
    #error "Float size must be 4 bytes"
  #endif
  #if _F0 == 0
    #define FINIT(w0, wx) {w0, wx} /* big-endian */
  #else
    #define FINIT(w0, wx) {wx, w0} /* little-endian */
  #endif
  const _Dvals _Flt = {
	(int)((FFRAC - 1) * LOG2),			/* FLT_DIG */
	(int)FFRAC,					/* FLT_MANT_DIG */
	(int)((FMAXE - _FBIAS - 1) * LOG2),		/* FLT_MAX_10_EXP */
	(int)(FMAXE - _FBIAS - 1),			/* FLT_MAX_EXP */
	(int)(-_FBIAS * LOG2),				/* FLT_MIN_10_EXP */
	(int)(1 - _FBIAS),				/* FLT_MIN_EXP */
	FINIT(_FBIAS - FFRAC + 2 << _FOFF, 0),	        /* FLT_EPSILON */
	FINIT((FMAXE << _FOFF) - 1, ONES),		/* FLT_MAX */
	FINIT(1 << _FOFF, 0),				/* FLT_MIN */
	};
#endif /* _FLOAT_IS_DOUBLE */

#if __DOUBLE_SIZE__ == 4
  #if _D0 == 0
    #define DINIT(w0, wx) {w0, wx} /* big-endian */
  #else
    #define DINIT(w0, wx) {wx, w0} /* little-endian */
  #endif
#elif __DOUBLE_SIZE__ == 8
  #if _D0 == 0
    #define DINIT(w0, wx) {w0, wx, wx, wx} /* big-endian */
  #else
    #define DINIT(w0, wx) {wx, wx, wx, w0} /* little-endian */
  #endif
#else
  #error "Double size must be 4 or 8 bytes"
#endif
const _Dvals _Dbl = {
	(int)((DFRAC - 1) * LOG2),			/* DBL_DIG */
	(int)DFRAC,					/* DBL_MANT_DIG */
	(int)((DMAXE - _DBIAS - 1) * LOG2),		/* DBL_MAX_10_EXP */
	(int)(DMAXE - _DBIAS - 1),			/* DBL_MAX_EXP */
	(int)(-_DBIAS * LOG2),				/* DBL_MIN_10_EXP */
	(int)(1 - _DBIAS),				/* DBL_MIN_EXP */
	DINIT(_DBIAS - DFRAC + 2 << _DOFF, 0),	        /* DBL_EPSILON */
	DINIT((DMAXE << _DOFF) - 1, ONES),		/* DBL_MAX */
	DINIT(1 << _DOFF, 0),				/* DBL_MIN */
	};

#ifndef _LONG_DOUBLE_IS_DOUBLE 
  #if _DLONG == 0 /* 4 or 8 bytes */
    #if __LONG_DOUBLE_SIZE__ == 4
      #if _L0 == 0
        #define LINIT(w0, wx) {w0, wx} /* big-endian */
      #else
        #define LINIT(w0, wx) {wx, w0} /* little-endian */
      #endif
    #elif __LONG_DOUBLE_SIZE__ == 8
      #if _L0 == 0
        #define LINIT(w0, wx) {w0, wx, wx, wx} /* big-endian */
      #else
        #define LINIT(w0, wx) {wx, wx, wx, w0} /* little-endian */
      #endif
    #else
      #error "Long double size must be 4 or 8 bytes when DLONG is 0"
    #endif
    const _Dvals _Ldbl = {	/* 64-bit */
	(int)((LFRAC - 1) * LOG2),			/* LDBL_DIG */
	(int)LFRAC,					/* LDBL_MANT_DIG */
	(int)((LMAXE - _LBIAS - 1) * LOG2),		/* LDBL_MAX_10_EXP */
	(int)(LMAXE - _LBIAS - 1),			/* LDBL_MAX_EXP */
	(int)(-_LBIAS * LOG2),				/* LDBL_MIN_10_EXP */
	(int)(1 - _LBIAS),				/* LDBL_MIN_EXP */
	LINIT(_LBIAS - LFRAC + 2 << _LOFF, 0),	        /* LDBL_EPSILON */
	LINIT((LMAXE << _LOFF) - 1, ONES),		/* LDBL_MAX */
	LINIT(1 << _LOFF, 0),				/* LDBL_MIN */
	};
  #endif /* _DLONG == 0 */
  #if _DLONG == 1 /* 10 bytes */
    #if _L0 == 0
      #define LINIT(w0, w1, wx) {w0, w1, wx, wx, wx} /* big-endian */
    #else
      #define LINIT(w0, w1, wx) {wx, wx, wx, w1, w0} /* big-endian */
    #endif
    const _Dvals _Ldbl = {	/* 80-bit */
	(int)((LFRAC - 1) * LOG2),			/* LDBL_DIG */
	(int)LFRAC,					/* LDBL_MANT_DIG */
	(int)((LMAXE - _LBIAS - 1) * LOG2),		/* LDBL_MAX_10_EXP */
	(int)(LMAXE - _LBIAS - 1),			/* LDBL_MAX_EXP */
	(int)(-_LBIAS * LOG2),				/* LDBL_MIN_10_EXP */
	(int)(1 - _LBIAS),				/* LDBL_MIN_EXP */
	LINIT(_LBIAS - LFRAC + 2, 0x8000, 0),	        /* LDBL_EPSILON */
	LINIT(LMAXE - 1, ONES, ONES),			/* LDBL_MAX */
	LINIT(1, 0x8000, 0),				/* LDBL_MIN */
	};

  #else /* 1 < _DLONG */ /* not supported */
    #define LINIT(w0, wx) xxx
    const _Dvals _Ldbl = {	/* 128-bit SPARC */
	(int)((LFRAC - 1) * LOG2),			/* LDBL_DIG */
	(int)LFRAC,					/* LDBL_MANT_DIG */
	(int)((LMAXE - _LBIAS - 1) * LOG2),		/* LDBL_MAX_10_EXP */
	(int)(LMAXE - _LBIAS - 1),			/* LDBL_MAX_EXP */
	(int)(-_LBIAS * LOG2),				/* LDBL_MIN_10_EXP */
	(int)(1 - _LBIAS),				/* LDBL_MIN_EXP */
	LINIT(_LBIAS - LFRAC + 2, 0, 0),		/* LDBL_EPSILON */
	LINIT(LMAXE - 1, ONES, ONES),			/* LDBL_MAX */
	LINIT(1, 0, 0),					/* LDBL_MIN */
	};
  #endif /* _DLONG */
#endif /* _LONG_DOUBLE_IS_DOUBLE */
_STD_END

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
