/* __xxx_GetMemChunk function */
/* Copyright (c) 2001-2003 by IAR Systems.  ALL RIGHTS RESERVED. */
#include "xalloc.h"
#include <stdlib.h>
#include <stdint.h>
_STD_BEGIN

/*
 * __xxx_GetMemChunk -- One for each memory that that privdes a heap.
 *
 * For non-paged memory types this simply returns a memory chunk that
 * contains the entire heap.  For paged memory this returs the memory
 * on one page at a time.  The last X bytes of a page is not included,
 * where X is the heap alignment.  This is for two reasons: 1) The
 * malloc help routine should not join two adjecent blocks in two
 * different memories.  2) Pointer to a location after an object
 * should not result in a pointer that points to the beginning of the
 * page.
 *
 * Returns 0 if there is no more memory.
 *
 * Note that the memory that we're returning is interpreted as an
 * __xxx_Cell structure, i.e. the first element is of type
 * __xxx_size_t and referes to the size of the block.
 *
 * No, there is no way to return memory to the heap, that is handled
 * internally by malloc and free.
 */

#pragma diag_suppress = Pe767  /* conversion from pointer to smaller integer */
#pragma diag_suppress = Pe1053 /* conversion from integer to smaller pointer */
#pragma diag_suppress = Pe236

#pragma language = extended

#define SEG(I) _GLUE3(__DATA_MEM,I,_HEAP_SEGMENT__)
#define HEAP_START(M, I) ((char const M *)__segment_begin(SEG(I)))
#define HEAP_END(M, I) ((char const M *)__segment_end(SEG(I)))
#define HEAP_END_ALIGNED(M, I) \
  ((char const M *)((M##_uintptr_t)HEAP_END(M,I) & ~(M##_uintptr_t)M_MASK))


/* page mask. E.g. for 64 Kbyte pages this macro is 0xFFFF.  Must be ("power of
 * two" - 1). */

#define P_MASK(I) _GLUE3(__DATA_MEM,I,_PAGE_SIZE__)
#define PRAGMA(X) _Pragma(#X)
#define PRAGMA3(X,Y,Z) PRAGMA(X Y Z)
#define PRAGMA4(X,Y,Z,P) PRAGMA(X Y Z P)

#define __HEAP_MEM_HELPER1__(M, I)                                      \
PRAGMA4(segment =, SEG(I), M, MAX_ALIGNMENT)                            \
void M * M##_GetMemChunk(void)                                          \
{                                                                       \
  static char const M * start = 0;                                      \
                                                                        \
  if (P_MASK(I) == 0)                                                   \
  {                                                                     \
    /* This is the non-paged case. */                                   \
    if (start != 0)                                                     \
    {                                                                   \
      return 0;                                                         \
    }                                                                   \
    start = HEAP_START(M, I);                                           \
    *(M##_size_t M *)start                                              \
      = HEAP_END_ALIGNED(M, I) - HEAP_START(M, I);                      \
    return (void M *)start;                                             \
  }                                                                     \
  else                                                                  \
  {                                                                     \
    /* Initialize our start pointer */                                  \
    if (start == 0)                                                     \
    {                                                                   \
      start = HEAP_START(M, I);                                         \
    }                                                                   \
                                                                        \
    {                                                                   \
      /* Get end of current page (minus alignment, since we             \
       * can't point to last byte.)  */                                 \
      /* Was " - 1" instead of "+ (P - M_MASK)". */                     \
      char const M * p =                                                \
        (  start                                                        \
         - ((M##_size_t)start & P_MASK(I))                              \
         + (P_MASK(I) - M_MASK));                                       \
      if ((M##_uintptr_t)p < (M##_uintptr_t)HEAP_END(M, I))             \
      {                                                                 \
        /* Allocate memory up to next end-of-page. */                   \
        char const M * r = start;                                       \
        *(M##_size_t M *)start = p - start;                             \
        start = (char const M *)((M##_uintptr_t)p + M_MASK + 1);        \
        return (void M *)r;                                             \
      }                                                                 \
      else if (start == HEAP_END(M, I))                                 \
      {                                                                 \
        return 0;                                                       \
      }                                                                 \
      else                                                              \
      {                                                                 \
        /* Found last chunk. */                                         \
        *(M##_size_t M *)start = (HEAP_END_ALIGNED(M, I) - start);      \
        p = start;                                                      \
        start = HEAP_END(M, I);                                         \
        return (void M *)p;                                             \
      }                                                                 \
    }                                                                   \
  }                                                                     \
}

__HEAP_MEMORY_LIST1__()
#pragma diag_default = Pe1053

#undef __HEAP_MEM_HELPER1__

_STD_END
