/* Tiny, handcrafted, printf. */
/* Copyright (C) 2003 IAR Systems.  All rights reserved. */
#include <string.h>
#include "xstdio.h"
_STD_BEGIN

#include <yfuns.h>

/* True if %ld should be supported. */
#ifndef _DLIB_PRINTFTINY_QUALIFIER_L
#define _DLIB_PRINTFTINY_QUALIFIER_L 0
#endif

/* Decide the integer type to use. */
#if _DLIB_PRINTFTINY_QUALIFIER_L || (__UINTPTR_T_MAX__ > __UNSIGNED_INT_MAX__)
#define _DLIB_PRINTFTINY_LONG 1
#else
#define _DLIB_PRINTFTINY_LONG 0
#endif

#if _DLIB_PRINTFTINY_LONG
typedef long int INT;
typedef unsigned long int UINT;
#else
typedef int INT;
typedef unsigned int UINT;
#endif


#if _DLIB_PRINTF_CHAR_BY_CHAR

#pragma inline
static void * out(_PrintfPfnType * pfn, void * arg, char const * p, int n)
{
  while (n)
  {
    arg = (*pfn)(arg, *p);
    ++p;
    --n;
  }
  return arg;
}

#define prout(p,n) (arg = out(pfn, arg, p, n))

#else /* _DLIB_PRINTF_CHAR_BY_CHAR */
#define prout(p,n) (arg = (*pfn)(arg, p, n))
#endif /* _DLIB_PRINTF_CHAR_BY_CHAR */


static char no2dig(char c, char code)
{
  c += '0';
  if (c > '9')
    c += (code - 'x') + 'a' - '9' - 1;

  return c;
}

#pragma diag_suppress = Pe236

int _PrintfTiny(_PrintfPfnType * pfn,
                void *arg,
                const char *fmt,
                va_list *pap)
{

  for (; ; )
  {       /* scan format string */
    for (; ; )
    {       /* copy any literal text */
      if (*fmt == '\0')
      {
        return 0;
      }
      else if (*fmt == '%')
      {
        ++fmt;
        break;
      }
      else
      {
        prout(fmt, 1);
      }
      fmt++;
    }
    {      /* do the conversion */
      INT value;

      char code = *fmt++;
#if _DLIB_PRINTFTINY_QUALIFIER_L
      int wasL = 0;
      if (code == 'l')
      {
        wasL = 1;
        code = *fmt++;
      }
#endif
      switch (code)
      {       /* switch on conversion specifier */
      case 'c':       /* convert a single character */
        {
          char c = va_arg(*pap, int);
          prout(&c, 1);
        }
        break;

      case 'd':     /* convert a signed decimal */
      case 'i':
#if _DLIB_PRINTFTINY_QUALIFIER_L
        value = wasL ? va_arg(*pap, long int) : va_arg(*pap, int);
#else
        value = va_arg(*pap, int);
#endif
        if (value < 0)       /* negate safely in _Litob */
          prout("-", 1);
_Litob:
        {
          char ac[16];
          INT const base = code == 'o' ? 8 :
            _LC(code) != 'x' ? 10 : 16;
          int sac = sizeof(ac);
          UINT ulval = value;

          if ((code == 'd') && value < 0)
            ulval = -ulval; /* safe against overflow */

          do
          {       /* convert digits */
            ac[--sac] = no2dig(ulval % base, code);
            ulval = ulval / (UINT) base;
          }
          while (0 < ulval && 0 < sac);

          prout(ac + sac, sizeof(ac) - sac);
        }
        break;

      case 'o': case 'u':
      case 'x': case 'X':     /* convert unsigned */
#if _DLIB_PRINTFTINY_QUALIFIER_L
        value = wasL ? va_arg(*pap, unsigned long int)
                     : va_arg(*pap, unsigned int);
#else
        value = va_arg(*pap, unsigned int);
#endif
        goto _Litob;

      case 'p':       /* convert a pointer, hex long version */
        value  = (INT) va_arg(*pap, void *);
        if (sizeof (void *) == sizeof (unsigned long) &&
            sizeof (unsigned long) < sizeof (INT))
          value &= ULONG_MAX;
        if (sizeof (void *) == sizeof (unsigned int) &&
            sizeof (unsigned int) < sizeof (INT))
          value &= UINT_MAX;
        code = 'x';
        goto _Litob;

      case 's':       /* convert a string */
        {
          char *s = va_arg(*pap, char *);
          prout(s, strlen(s));
        }
        break;

      case '%':       /* put a '%' */
        prout("%", 1);
        break;
      }
    }
  }
}
_STD_END

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
