/* _Putfld function */
#include <string.h>
#include <wchar.h>
#include "xmath.h"
#include "xstdio.h"
_STD_BEGIN

int _Putfld(_Pft *px, va_list *pap, char code)
{       /* convert a field for _Printf */
  char *pac = px->ac;
  _PrintfSignedIntType li;

  switch (code)
  {       /* switch on conversion specifier */
  case 'c':       /* convert a single character */
#if _DLIB_PRINTF_MULTIBYTE && _DLIB_PRINTF_QUALIFIERS
    if (px->qual != 'l')
#endif
      pac[px->n0++] = va_arg(*pap, int);
#if _DLIB_PRINTF_MULTIBYTE && _DLIB_PRINTF_QUALIFIERS
    else
    {       /* convert as wide string */
      wchar_t wc[2];
 #if WCHAR_MAX <= INT_MAX
			wint_t wi = (wint_t)va_arg(*pap, int);
 #else /* WCHAR_MAX <= INT_MAX */
      wint_t wi = va_arg(*pap, wint_t);
 #endif /* WCHAR_MAX <= INT_MAX */

      wc[0] = wi, wc[1] = L'\0';
#if _DLIB_PRINTF_WIDTH_AND_PRECISION
      px->prec = -1;
#endif
      if (_Putstr(px, (const wchar_t *)wc) < 0)
        return (EOF);
    }
#endif /* _DLIB_PRINTF_MULTIBYTE && _DLIB_PRINTF_QUALIFIERS */
    break;

#if _DLIB_PRINTF_SPECIFIER_SIGNED
  case 'd': case 'i':     /* convert a signed decimal */
#if _DLIB_PRINTF_QUALIFIERS
    li = px->qual == 'l' ? va_arg(*pap, long)
#if _DLIB_PRINTF_LONG_LONG
        : px->qual == 'q' ? va_arg(*pap, _Longlong)
#endif
#if _PRINTF_INTERNAL_QUALIFIER_J
        : px->qual == 'j' ? va_arg(*pap, intmax_t)
#endif
      : va_arg(*pap, int);
    if (px->qual == 'h')
      li = (short)li;
    else if (px->qual == 'b')
      li = (signed char)li;
    else if (px->qual == 't' || px->qual == 'z')
      li = (ptrdiff_t)li;
#else
    li = va_arg(*pap, int);
#endif /* _DLIB_PRINTF_QUALIFIERS */
    if (li < 0)       /* negate safely in _Litob */
      pac[px->n0++] = '-';
#if _DLIB_PRINTF_FLAGS
    else if (px->flags & _FPL)
      pac[px->n0++] = '+';
    else if (px->flags & _FSP)
      pac[px->n0++] = ' ';
#endif /* _DLIB_PRINTF_FLAGS */
    px->s = &pac[px->n0];
    px->v.li = li;
    _Litob(px, code);
    break;
#endif /* _DLIB_PRINTF_SPECIFIER_SIGNED */

#if _DLIB_PRINTF_SPECIFIER_UNSIGNED
  case 'o': case 'u':
  case 'x': case 'X':     /* convert unsigned */
#if _DLIB_PRINTF_QUALIFIERS
    li = px->qual == 'l' ? va_arg(*pap, unsigned long)
      : px->qual == 'q' ? va_arg(*pap, _ULonglong)
      : px->qual == 'j' ? va_arg(*pap, uintmax_t)
      : va_arg(*pap, unsigned int);
    if (px->qual == 'h')
      li = (unsigned short)li;
    else if (px->qual == 'b')
      li = (unsigned char)li;
    else if (px->qual == 't' || px->qual == 'z')
      li = (size_t)li;
#else
    li = va_arg(*pap, unsigned int);
#endif /* _DLIB_PRINTF_QUALIFIERS */
#if _DLIB_PRINTF_FLAGS
    if (px->flags & _FNO && li != 0
        && (_LC(code) == 'x'))
      pac[px->n0++] = '0', pac[px->n0++] = code;
#endif /* _DLIB_PRINTF_FLAGS */
    px->s = &pac[px->n0];
    px->v.li = li;
    _Litob(px, code);
    break;
#endif /* _DLIB_PRINTF_SPECIFIER_UNSIGNED */

#if _DLIB_PRINTF_SPECIFIER_FLOAT
  case 'e': case 'E':     /* convert floating */
  case 'g': case 'G':
  case 'f': case 'F':
#if _DLIB_PRINTF_SPECIFIER_A
  case 'a': case 'A':
#endif
#if _DLIB_PRINTF_QUALIFIERS
    px->v.ld = px->qual == 'L'
      ? va_arg(*pap, long double) : va_arg(*pap, double);
#else
    px->v.ld = va_arg(*pap, double);
#endif /* _DLIB_PRINTF_QUALIFIERS */
    if (LSIGN(px->v.ld))
      pac[px->n0++] = '-';
#if _DLIB_PRINTF_FLAGS
    else if (px->flags & _FPL)
      pac[px->n0++] = '+';
    else if (px->flags & _FSP)
      pac[px->n0++] = ' ';
#endif /* _DLIB_PRINTF_FLAGS */
    px->s = &pac[px->n0];
    _Ldtob(px, code);
    break;
#endif /* _DLIB_PRINTF_SPECIFIER_FLOAT */

#if _DLIB_PRINTF_SPECIFIER_N
  case 'n':       /* return output count */
#if _DLIB_PRINTF_QUALIFIERS
    switch (px->qual)
    {       /* store in specified integer type */
    case 'b':
      *va_arg(*pap, signed char *) = px->nchar;
      break;

#if _DLIB_PRINTF_LONG_LONG
    case 'q':
      *va_arg(*pap, _Longlong *) = px->nchar;
      break;
#endif

#if _PRINTF_INTERNAL_QUALIFIER_J
    case 'j':
      *va_arg(*pap, intmax_t *) = px->nchar;
      break;
#endif

    case 't':
      *va_arg(*pap, ptrdiff_t *) = px->nchar;
      break;

    case 'z':
      *va_arg(*pap, size_t *) = px->nchar;
      break;

    case 'h':
      *va_arg(*pap, short *) = px->nchar;
      break;

    case 'l':
      *va_arg(*pap, long *) = px->nchar;
      break;

    default:
      *va_arg(*pap, int *) = px->nchar;
    }
#else
    *va_arg(*pap, int *) = px->nchar;
#endif /* _DLIB_PRINTF_QUALIFIERS */
    break;
#endif /* _DLIB_PRINTF_SPECIFIER_N */

  case 'p':       /* convert a pointer, hex long version */
    li = (_PrintfSignedIntType) va_arg(*pap, void *);
    if (sizeof (void *) == sizeof (unsigned long) &&
        sizeof (unsigned long) < sizeof (_PrintfSignedIntType))
      li &= ULONG_MAX;
    if (sizeof (void *) == sizeof (unsigned int) &&
        sizeof (unsigned int) < sizeof (_PrintfSignedIntType))
      li &= UINT_MAX;
    px->s = pac;
    px->v.li = li;
    _Litob(px, 'x');
    break;

  case 's':       /* convert a string */
#if _DLIB_PRINTF_MULTIBYTE && _DLIB_PRINTF_QUALIFIERS
    if (px->qual != 'l')
#endif
    {       /* determine length safely */
      px->s = va_arg(*pap, char *);

#if _DLIB_PRINTF_WIDTH_AND_PRECISION
      {
        char *s1;
        px->n1 = px->prec < 0 ? strlen(px->s)
          : (s1 = (char *)memchr((void *)px->s, '\0',
                                 px->prec)) != 0 ? s1 - px->s : px->prec;
      }
#else
      px->n1 = strlen(px->s);
#endif
    }
#if _DLIB_PRINTF_MULTIBYTE && _DLIB_PRINTF_QUALIFIERS
    else if (_Putstr(px,
                     va_arg(*pap, const wchar_t *)) < 0)
      return (EOF);
#endif
    break;

  default:        /* undefined specifier or '%', print it out */
    pac[px->n0++] = code != '\0' ? code : '%';
  }
  return (0);
}
_STD_END

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
