/* xsyslock.c -- thread locking and unlocking functions */
#include <stdio.h>
#include "xmtx.h"
_STD_BEGIN

 #if !_MULTI_THREAD

 #else /* _MULTI_THREAD */
  #define _MAX_FLOCK	FOPEN_MAX

static _Rmtx file_mtx[_MAX_FLOCK];
static _Rmtx mtx[_MAX_LOCK];
static _Once_t syslock_o = _ONCE_T_INIT;

static void clearlocks(void)
	{	/* clear all locks at program termination */
	int count;

	for (count = 0; count < _MAX_FLOCK; ++count)
		_Mtxdst(&file_mtx[count]);
	for (count = 0; count < _MAX_LOCK; ++count)
		_Mtxdst(&mtx[count]);
	}

static void initlocks(void)
	{	/* initialize all locks on first call to _Locksyslock */
	int count;

	for (count = 0; count < _MAX_FLOCK; ++count)
		_Mtxinit(&file_mtx[count]);
	for (count = 0; count < _MAX_LOCK; ++count)
		_Mtxinit(&mtx[count]);
	_CSTD _Atexit(&clearlocks);
	}

void (_Lockfilelock)(FILE *str)
	{	/* set a file lock */
	_Once(&syslock_o, initlocks);
	if (str != 0 && str->_Lockno < _MAX_FLOCK)
		_Mtxlock(&file_mtx[str->_Lockno]);
	}

void (_Unlockfilelock)(FILE *str)
	{	/* clear a system lock */
	if (str != 0 && str->_Lockno < _MAX_FLOCK)
		_Mtxunlock(&file_mtx[str->_Lockno]);
	}

void (_Locksyslock)(unsigned int lockno)
	{	/* set a system lock */
	_Once(&syslock_o, initlocks);
	if (lockno < _MAX_LOCK)
		_Mtxlock(&mtx[lockno]);
	}

void (_Unlocksyslock)(unsigned int lockno)
	{	/* clear a system lock */
	if (lockno < _MAX_LOCK)
		_Mtxunlock(&mtx[lockno]);
	}
 #endif /* _MULTI_THREAD */
_STD_END

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
